import * as THREE from 'three';
import React, { useRef, useMemo } from 'react';
import { extend, useThree, useLoader, useFrame } from '@react-three/fiber';
import { Water } from 'three-stdlib';

extend({
  Water
});
function Ocean() {
  const ref = useRef();
  const gl = useThree(state => state.gl);
  const waterNormals = useLoader(THREE.TextureLoader, 'https://ghcdn.rawgit.org/pmndrs/market-assets/components/files/components/src/water/waternormals.jpeg');
  waterNormals.wrapS = waterNormals.wrapT = THREE.RepeatWrapping;
  const geom = useMemo(() => new THREE.PlaneGeometry(10000, 10000), []);
  const config = useMemo(() => ({
    textureWidth: 512,
    textureHeight: 512,
    waterNormals,
    sunDirection: new THREE.Vector3(),
    sunColor: 0xffffff,
    waterColor: 0x001e0f,
    distortionScale: 3.7,
    fog: false,
    format: gl.encoding
  }), [waterNormals]);
  useFrame((state, delta) => ref.current.material.uniforms.time.value += delta);
  return /*#__PURE__*/React.createElement("water", {
    ref: ref,
    args: [geom, config],
    "rotation-x": -Math.PI / 2
  });
}

export { Ocean };
